@@ -3,39 +3,51 @@ use crate::data::handler::{Continuation, Continuer, MarketGenerator};
 use crate::data::market::MarketEvent;
 use barter_data::model::{Candle, MarketData};
 use chrono::Utc;
-use std::vec::IntoIter;
 use uuid::Uuid;
 
 /// Configuration for constructing a [HistoricCandleHandler] via the new() constructor method.
 #[derive(Debug)]
-pub struct HistoricDataLego {
+pub struct HistoricDataLego<Candles>
+where
+    Candles: Iterator<Item = Candle>
+{
     pub exchange: &'static str,
     pub symbol: String,
-    pub candle_iterator: IntoIter<Candle>,
+    pub candles: Candles,
 }
 
 #[derive(Debug)]
 /// [MarketEvent] data handler that implements [Continuer] & [MarketGenerator]. Simulates a live market
 /// feed via drip feeding historical data files as a series of [MarketEvent]s.
-pub struct HistoricCandleHandler {
+pub struct HistoricCandleHandler<Candles>
+where
+    Candles: Iterator<Item = Candle>
+{
     exchange: &'static str,
     symbol: String,
-    candle_iterator: IntoIter<Candle>,
+    can_continue: Continuation,
+    candles: Candles,
 }
 
-impl Continuer for HistoricCandleHandler {
+impl<Candles> Continuer for HistoricCandleHandler<Candles>
+where
+    Candles: Iterator<Item = Candle>
+{
     fn can_continue(&self) -> &Continuation {
-        match self.candle_iterator.len() != 0 {
-            true => &Continuation::Continue,
-            false => &Continuation::Stop,
-        }
+        &self.can_continue
     }
 }
 
-impl MarketGenerator for HistoricCandleHandler {
+impl<Candles> MarketGenerator for HistoricCandleHandler<Candles>
+where
+    Candles: Iterator<Item = Candle>
+{
     fn generate_market(&mut self) -> Option<MarketEvent> {
-        match self.candle_iterator.next() {
-            None => None,
+        match self.candles.next() {
+            None => {
+                self.can_continue = Continuation::Stop;
+                None
+            },
             Some(candle) => Some(MarketEvent {
                 event_type: MarketEvent::EVENT_TYPE,
                 trace_id: Uuid::new_v4(),
@@ -48,34 +60,49 @@ impl MarketGenerator for HistoricCandleHandler {
     }
 }
 
-impl HistoricCandleHandler {
+impl<Candles> HistoricCandleHandler<Candles>
+where
+    Candles: Iterator<Item = Candle>
+{
     /// Constructs a new [HistoricCandleHandler] component using the provided [HistoricDataLego]
     /// components.
-    pub fn new(lego: HistoricDataLego) -> Self {
+    pub fn new(lego: HistoricDataLego<Candles>) -> Self
+    {
         Self {
             exchange: lego.exchange,
             symbol: lego.symbol,
-            candle_iterator: lego.candle_iterator,
+            can_continue: Continuation::Continue,
+            candles: lego.candles,
         }
     }
 
     /// Returns a [HistoricCandleHandlerBuilder] instance.
-    pub fn builder() -> HistoricCandleHandlerBuilder {
+    pub fn builder() -> HistoricCandleHandlerBuilder<Candles> {
         HistoricCandleHandlerBuilder::new()
     }
 }
 
 /// Builder to construct [HistoricCandleHandler] instances.
 #[derive(Debug, Default)]
-pub struct HistoricCandleHandlerBuilder {
+pub struct HistoricCandleHandlerBuilder<Candles>
+where
+    Candles: Iterator<Item = Candle>
+{
     exchange: Option<&'static str>,
     symbol: Option<String>,
-    candle_iterator: Option<IntoIter<Candle>>,
+    candles: Option<Candles>,
 }
 
-impl HistoricCandleHandlerBuilder {
+impl<Candles> HistoricCandleHandlerBuilder<Candles>
+where
+    Candles: Iterator<Item = Candle>
+{
     pub fn new() -> Self {
-        Self::default()
+        Self {
+            exchange: None,
+            symbol: None,
+            candles: None
+        }
     }
 
     pub fn symbol(self, value: String) -> Self {
@@ -92,22 +119,23 @@ impl HistoricCandleHandlerBuilder {
         }
     }
 
-    pub fn candle_iterator(self, value: IntoIter<Candle>) -> Self {
+    pub fn candles(self, value: Candles) -> Self {
         Self {
-            candle_iterator: Some(value),
+            candles: Some(value),
             ..self
         }
     }
 
-    pub fn build(self) -> Result<HistoricCandleHandler, DataError> {
+    pub fn build(self) -> Result<HistoricCandleHandler<Candles>, DataError> {
         let exchange = self.exchange.ok_or(DataError::BuilderIncomplete)?;
         let symbol = self.symbol.ok_or(DataError::BuilderIncomplete)?;
-        let candle_iterator = self.candle_iterator.ok_or(DataError::BuilderIncomplete)?;
+        let candles = self.candles.ok_or(DataError::BuilderIncomplete)?;
 
         Ok(HistoricCandleHandler {
             exchange,
             symbol,
-            candle_iterator,
+            can_continue: Continuation::Continue,
+            candles,
         })
     }
 }
@@ -124,7 +152,7 @@ mod tests {
         let data_handler = HistoricCandleHandler::builder()
             .exchange("Backtest")
             .symbol("DOGE".to_string())
-            .candle_iterator(symbol_data_remaining.into_iter())
+            .candles(symbol_data_remaining.into_iter())
             .build()
             .unwrap();
 
@@ -137,13 +165,15 @@ mod tests {
     fn should_not_continue_with_no_symbol_data_remaining() {
         let symbol_data_remaining: Vec<Candle> = Vec::with_capacity(2);
 
-        let data_handler = HistoricCandleHandler::builder()
+        let mut data_handler = HistoricCandleHandler::builder()
             .exchange("Backtest")
             .symbol("DOGE".to_string())
-            .candle_iterator(symbol_data_remaining.into_iter())
+            .candles(symbol_data_remaining.into_iter())
             .build()
             .unwrap();
 
+        data_handler.generate_market();
+
         let actual_can_continue = data_handler.can_continue();
 
         assert_eq!(actual_can_continue, &Continuation::Stop);
